@@ -1,9 +1,9 @@
 """
-SIRIUS functions
+SIRIUS
 """
 #__init__.py
 from sirius._version import __version__
-from .calc_a_noise import calc_a_noise_chunk
+from .calc_noise import calc_noise_chunk
 from .calc_vis import calc_vis_chunk
 from .calc_uvw import calc_uvw_chunk
 from .calc_beam import calc_zpc_beam, evaluate_beam_models
